# pylint: disable=duplicate-code
PLAYER_INPUT = 1
PROJECTILE = 2
LASER = 3
PICKUP = 4
FLAG = 5
GAME_INFO = 6
GAME_DATA = 7
CHARACTER_CORE = 8
CHARACTER = 9
PLAYER_INFO = 10
CLIENT_INFO = 11
SPECTATOR_INFO = 12
COMMON = 13
EXPLOSION = 14
SPAWN = 15
HAMMERHIT = 16
DEATH = 17
SOUND_GLOBAL = 18
SOUND_WORLD = 19
DAMAGE_INDICATOR = 20
